/*
 * irreco - Ir Remote Control
 * Copyright (C) 2008 Joni Kokko (t5kojo01@students.oamk.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "irreco_theme.h"

/**
 * @addtogroup IrrecoTheme
 * @ingroup Irreco
 *
 * Contains information of theme.
 *
 * @{ 
 */

/**
 * @file
 * Source file of @ref IrrecoTheme.
 */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void irreco_theme_read_button_keyfile_foreach(IrrecoDirForeachData * dir_data);
void irreco_theme_read_bg_keyfile_foreach(IrrecoDirForeachData * dir_data);
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Construction & Destruction                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Construction & Destruction
 * @{
 */

IrrecoTheme *irreco_theme_new(const char *name, const char *path,
			      const char *source, const char *author,
			      const char *comment,
			      const char *preview_button_name,
			      const char *version)
{
	IrrecoTheme *self;
	IRRECO_ENTER
	
	self = g_slice_new0(IrrecoTheme);
	
	self->name = g_string_new(name);
	self->path = g_string_new(path);
	self->source = g_string_new(source);
	self->author = g_string_new(author);
	self->comment = g_string_new(comment);
	self->preview_button_name = g_string_new(preview_button_name);
	self->version = g_string_new(version);
	self->backgrounds =  irreco_string_table_new(
				(GDestroyNotify)irreco_theme_bg_free, NULL);
	self->buttons =  irreco_string_table_new(
				(GDestroyNotify)irreco_theme_button_free, NULL);
	
	IRRECO_DEBUG("Themepath: %s\n", path);

	/*Get buttons*/
	{
		IrrecoDirForeachData button_styles;
		GString * directory = g_string_new("");	
	
		g_string_printf(directory, "%s/buttons/", path);
		IRRECO_DEBUG("Directory = %s\n", directory->str);
		button_styles.directory = directory->str;
	
		button_styles.filesuffix = "button.conf";
		button_styles.user_data_1 = self;
	
		irreco_dir_foreach_subdirectories(&button_styles, 
				irreco_theme_read_button_keyfile_foreach);

		g_string_free(directory, TRUE);
		directory = NULL;

		irreco_string_table_sort_abc(self->buttons);
	}
	
	/*Get backgrounds*/
	{
		IrrecoDirForeachData bg_styles;
		GString * directory = g_string_new("");	
	
		g_string_printf(directory, "%s/bg/", path);
		IRRECO_DEBUG("Directory = %s\n", directory->str);
		bg_styles.directory = directory->str;

		bg_styles.filesuffix = "bg.conf";
		bg_styles.user_data_1 = self;

		irreco_dir_foreach_subdirectories(&bg_styles,
				irreco_theme_read_bg_keyfile_foreach);

		g_string_free(directory, TRUE);
		directory = NULL;

		irreco_string_table_sort_abc(self->backgrounds);
	}

	IRRECO_RETURN_PTR(self);
}

void irreco_theme_free(IrrecoTheme *self)
{
	IRRECO_ENTER

	g_assert(self != NULL);

	g_string_free(self->name, TRUE);
	self->name = NULL;

	g_string_free(self->path, TRUE);
	self->path = NULL;

	g_string_free(self->source, TRUE);
	self->source = NULL;

	g_string_free(self->author, TRUE);
	self->author = NULL;

	g_string_free(self->comment, TRUE);
	self->comment = NULL;

	g_string_free(self->preview_button_name, TRUE);
	self->preview_button_name = NULL;

	irreco_string_table_free(self->backgrounds);
	self->backgrounds = NULL;

	irreco_string_table_free(self->buttons);
	self->buttons = NULL;

	g_slice_free(IrrecoTheme, self);
	self = NULL;

	IRRECO_RETURN
}

/** @} */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Private Functions                                                          */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Private Functions
 * @{
 */


/** @} */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Public Functions                                                           */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Public Functions
 * @{
 */
void irreco_theme_update_keyfile(IrrecoTheme *self)
{
	GString *keyfile_path = g_string_new(self->path->str);
	GKeyFile *keyfile = g_key_file_new();
	IRRECO_ENTER

	g_string_append(keyfile_path, "/theme.conf");

	irreco_gkeyfile_set_string(keyfile, "theme" , "name",
				   self->name->str);

	if (self->source->len > 0) {
		irreco_gkeyfile_set_string(keyfile, "theme" , "source",
					   self->source->str);
	}

	if (self->author->len > 0) {
		irreco_gkeyfile_set_string(keyfile, "theme", "author",
					   self->author->str);
	}

	if (self->preview_button_name->len > 0) {
		irreco_gkeyfile_set_string(keyfile, "theme", "preview-button",
					   self->preview_button_name->str);
	}

	if (self->version->len > 0) {
		irreco_gkeyfile_set_string(keyfile, "theme", "version",
					   self->version->str);
	}

	if (self->comment->len > 0) {
		irreco_gkeyfile_set_string(keyfile, "theme", "comment",
					   self->comment->str);
	}

	irreco_write_keyfile(keyfile, keyfile_path->str);

	g_key_file_free(keyfile);
	g_string_free(keyfile_path, TRUE);
	IRRECO_RETURN
}

void irreco_theme_print(IrrecoTheme *self)
{
	IRRECO_ENTER

	IRRECO_DEBUG("Themename: %s \n", self->name->str);
	IRRECO_DEBUG("Folder: %s \n", self->path->str);
	IRRECO_DEBUG("Source: %s \n", self->source->str);
	IRRECO_DEBUG("Author: %s \n", self->author->str);
	IRRECO_DEBUG("Comment: %s \n", self->comment->str);
	IRRECO_DEBUG("Previewbutton: %s \n", self->preview_button_name->str);
	IRRECO_DEBUG("Version: %s \n", self->version->str);
	irreco_string_table_print(self->backgrounds);
	irreco_string_table_print(self->buttons);
	
	#if 0
	IRRECO_STRING_TABLE_FOREACH_DATA(self->backgrounds, void *, theme_bg)
		irreco_theme_button_print(theme_bg);
	IRRECO_STRING_TABLE_FOREACH_END

	IRRECO_STRING_TABLE_FOREACH_DATA(self->buttons, void *, theme_button)
		irreco_theme_button_print(theme_button);
	IRRECO_STRING_TABLE_FOREACH_END
	#endif
	IRRECO_RETURN
}

void irreco_theme_read_button_keyfile_foreach(IrrecoDirForeachData * dir_data)
{
	IrrecoTheme	*self = (IrrecoTheme*) dir_data->user_data_1;
	IrrecoKeyFile	*keyfile;
	gchar *name = NULL;
	gchar *up = NULL;
	gchar *down = NULL;
	gboolean allow_text;
	gchar *text_format_up = NULL;
	gchar *text_format_down = NULL;
	gint text_padding = 5;
	gfloat text_h_align = 0.5;
	gfloat text_v_align = 0.5;
	IrrecoThemeButton *theme_button;
	GString	*style_name = g_string_new(self->name->str);

	IRRECO_ENTER

	keyfile = irreco_keyfile_create(dir_data->directory,
					dir_data->filepath,
					"theme-button");
	if (keyfile == NULL) goto end;
	
	/* Required fields. */			
	if (!irreco_keyfile_get_str(keyfile, "name", &name) ||
	    !irreco_keyfile_get_path(keyfile, "up", &up)) {
		IRRECO_PRINTF("Could not read style from group \"%s\"\n", 
			      keyfile->group);
		goto end;
	}

	/* Optional fields. */
	irreco_keyfile_get_path(keyfile, "down", &down);
	irreco_keyfile_get_bool(keyfile, "allow-text", &allow_text);
	irreco_keyfile_get_str(keyfile, "text-format-up", &text_format_up);
	irreco_keyfile_get_str(keyfile, "text-format-down", &text_format_down);
	irreco_keyfile_get_int(keyfile, "text-padding", &text_padding);
	irreco_keyfile_get_float(keyfile, "text-h-align", &text_h_align);
	irreco_keyfile_get_float(keyfile, "text-v-align", &text_v_align);
	
	g_string_append_printf(style_name,"/%s", name);
	theme_button = irreco_theme_button_new();
	irreco_theme_button_set(theme_button, style_name->str,
				name, allow_text,
				up, down,
				text_format_up, text_format_down,
				text_padding,
				text_h_align,
				text_v_align);

	irreco_string_table_add(self->buttons,
				theme_button->style_name->str,
				theme_button);

	irreco_theme_button_print(theme_button);

	end:
	if(keyfile != NULL) irreco_keyfile_destroy(keyfile);
	if(name != NULL) g_free(name);
	if(name != NULL) g_free(up);
	if(name != NULL) g_free(down);
	if(name != NULL) g_free(text_format_up);
	if(name != NULL) g_free(text_format_down);
	g_string_free(style_name, TRUE);


	IRRECO_RETURN
}

void irreco_theme_read_bg_keyfile_foreach(IrrecoDirForeachData * dir_data)
{
	IrrecoTheme	*self = (IrrecoTheme*) dir_data->user_data_1;
	IrrecoKeyFile	*keyfile;
	IrrecoThemeBg	*bg;
	char		*name	= NULL;
	char		*image	= NULL;
	IRRECO_ENTER

	keyfile = irreco_keyfile_create(dir_data->directory,
					dir_data->filepath,
					"theme-bg");
	if (keyfile == NULL) goto end;

	/* Required fields. */
	if (!irreco_keyfile_get_str(keyfile, "name", &name) ||
	    !irreco_keyfile_get_path(keyfile, "image", &image)) {
		IRRECO_PRINTF("Could not read style from group \"%s\"\n",
			      keyfile->group);
		goto end;
	}
	bg = irreco_theme_bg_new();

	irreco_theme_bg_set(bg, name, image);
	irreco_string_table_add(self->backgrounds, name, bg);

	irreco_theme_bg_print(bg);

	end:
	if(keyfile != NULL) irreco_keyfile_destroy(keyfile);
	if(name != NULL) g_free(name);
	if(image != NULL) g_free(image);
	IRRECO_RETURN
}

IrrecoStringTable* irreco_theme_get_buttons(IrrecoTheme *self)
{
	IRRECO_ENTER		
	IRRECO_RETURN_PTR(self->buttons);
}

IrrecoThemeButton *irreco_theme_get_button(IrrecoTheme *self,
					   const char *button_name)
{
	IrrecoThemeButton *button = NULL;
	IRRECO_ENTER
	IRRECO_STRING_TABLE_FOREACH_DATA(self->buttons, IrrecoThemeButton *,
					 pointer)
		if (g_utf8_collate(pointer->name->str, button_name) == 0) {
			button = pointer;
		}
	IRRECO_STRING_TABLE_FOREACH_END
	IRRECO_RETURN_PTR(button);
}

IrrecoStringTable* irreco_theme_get_backgrounds(IrrecoTheme *self)
{
	IRRECO_ENTER		
	IRRECO_RETURN_PTR(self->backgrounds);
}

IrrecoThemeBg *irreco_theme_get_background(IrrecoTheme *self,
					   const char *bg_name)
{
	IrrecoThemeBg *bg = NULL;
	IRRECO_ENTER
	IRRECO_STRING_TABLE_FOREACH_DATA(self->backgrounds, IrrecoThemeBg *,
					 pointer)
		if (g_utf8_collate(pointer->image_name->str, bg_name) == 0) {
			bg = pointer;
		}
	IRRECO_STRING_TABLE_FOREACH_END
	IRRECO_RETURN_PTR(bg);
}

void irreco_theme_set_author(IrrecoTheme *self, const char * author)
{
	IRRECO_ENTER
	if (author != NULL) {
		g_string_printf(self->author, "%s", author);

		irreco_theme_update_keyfile(self);
	}
	IRRECO_RETURN
}

void irreco_theme_set_comment(IrrecoTheme *self, const char * comment)
{
	IRRECO_ENTER
	if (comment != NULL) {
		g_string_printf(self->comment, "%s", comment);

		irreco_theme_update_keyfile(self);
	}
	IRRECO_RETURN
}

void irreco_theme_set_preview_button(IrrecoTheme *self,
				     const char * button_name)
{
	IRRECO_ENTER
	if (button_name != NULL) {
		g_string_printf(self->preview_button_name, "%s", button_name);

		irreco_theme_update_keyfile(self);
	}
	IRRECO_RETURN
}

#if 0
/* This function will work after IrrecoButtonStyle destruction*/

void irreco_theme_set_name(IrrecoTheme *self, IrrecoData *irreco_data,
			   const char * name)
{
	IRRECO_ENTER
	if (name != NULL) {
		GString	*style_name = g_string_new("");

		/*
		 *TODO Move this part to IrrecoThemeManager and call this
		 *funtion from ThemeManager*/
		{
		irreco_string_table_change_key(
					irreco_data->theme_manager->themes,
					self->name->str, name);
		}
		g_string_printf(self->name, "%s", name);

		IRRECO_STRING_TABLE_FOREACH_DATA(self->buttons,
						 IrrecoThemeButton *, button) {
			g_string_printf(button->style_name,"%s/%s",
					self->name->str,
					button->name->str);
			IRRECO_PRINTF("style: %s\n",button->style_name->str);
			IRRECO_PAUSE
		}
		IRRECO_STRING_TABLE_FOREACH_END

		irreco_theme_update_keyfile(self);
		irreco_config_save_layouts(irreco_data);
		g_string_free(style_name, TRUE);
	}
	IRRECO_RETURN
}
#endif
/** @} */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Events and Callbacks                                                       */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/** @} */
